==> liquidwar6-0.0.1beta.tar.gz: liquidwar6-0.0.1beta/src/lib/map/map-resampler.c <==
/*
Liquid War 6 is a unique multiplayer wargame.
Copyright (C)  2005, 2006  Christian Mauduit <ufoot@ufoot.org>

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

Liquid War 6 homepage : http://www.gnu.org/software/liquidwar6/
Contact author        : ufoot@ufoot.org
*/

#include <stdlib.h>
#include <string.h>
#include <math.h>

#include "config.h"
#include "map.h"
#include "map-internal.h"

void
lw6map_resampler_init (LW6MAP_RESAMPLER * resampler,
		       LW6OPT_STATIC * static_options,
		       LW6SYS_WH * source_shape)
{
  LW6SYS_WH target_shape;
  float f;

  memset (resampler, 0, sizeof (LW6MAP_RESAMPLER));

  target_shape = (*source_shape);

  if (target_shape.w * target_shape.h > static_options->max_map_surface)
    {
      f =
	sqrt ((((float) target_shape.w) * ((float) target_shape.h)) /
	      ((float) static_options->max_map_surface));
      target_shape.w /= f;
      target_shape.h /= f;
    }

  if (target_shape.w * target_shape.h < static_options->min_map_surface)
    {
      f =
	sqrt (((float) static_options->min_map_surface) /
	      (((float) target_shape.w) * ((float) target_shape.h)));
      target_shape.w *= f;
      target_shape.h *= f;
    }

  target_shape.w = lw6sys_min (target_shape.w, static_options->max_map_width);
  target_shape.h =
    lw6sys_min (target_shape.h, static_options->max_map_height);

  target_shape.w = lw6sys_max (target_shape.w, static_options->min_map_width);
  target_shape.h =
    lw6sys_max (target_shape.h, static_options->min_map_height);

  resampler->target_shape = target_shape;
  resampler->source_shape = (*source_shape);
  resampler->scale_x =
    ((float) (resampler->target_shape.w)) /
    ((float) (resampler->source_shape.w));
  resampler->scale_y =
    ((float) (resampler->target_shape.h)) /
    ((float) (resampler->source_shape.h));
}

static void
check_bounds (int *x, int *y, LW6SYS_WH * shape)
{
  (*x) = lw6sys_min ((*x), shape->w - 1);
  (*y) = lw6sys_min ((*y), shape->h - 1);
  (*x) = lw6sys_max ((*x), 0);
  (*y) = lw6sys_max ((*y), 0);
}

void
lw6map_resampler_source2target (LW6MAP_RESAMPLER * resampler, int *target_x,
				int *target_y, int source_x, int source_y)
{
  (*target_x) = (int) (source_x * resampler->scale_x);
  (*target_y) = (int) (source_y * resampler->scale_y);
  check_bounds (target_x, target_y, &(resampler->target_shape));
}

void
lw6map_resampler_target2source (LW6MAP_RESAMPLER * resampler, int *source_x,
				int *source_y, int target_x, int target_y)
{
  (*source_x) = (int) (target_x / resampler->scale_x);
  (*source_y) = (int) (target_y / resampler->scale_y);
  check_bounds (source_x, source_y, &(resampler->source_shape));
}

==> liquidwar6-0.0.2beta.tar.gz: liquidwar6-0.0.2beta/src/lib/map/map-resampler.c <==
/*
Liquid War 6 is a unique multiplayer wargame.
Copyright (C)  2005, 2006, 2007  Christian Mauduit <ufoot@ufoot.org>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.


Liquid War 6 homepage : http://www.gnu.org/software/liquidwar6/
Contact author        : ufoot@ufoot.org
*/

#include <stdlib.h>
#include <string.h>
#include <math.h>

#include "config.h"
#include "map.h"
#include "map-internal.h"

void
lw6map_resampler_init (LW6MAP_RESAMPLER * resampler,
		       LW6OPT_STATIC * static_options,
		       LW6SYS_WH * source_shape)
{
  LW6SYS_WH target_shape;
  float f;

  memset (resampler, 0, sizeof (LW6MAP_RESAMPLER));

  target_shape = (*source_shape);

  if (target_shape.w * target_shape.h > static_options->max_map_surface)
    {
      f =
	sqrt ((((float) target_shape.w) * ((float) target_shape.h)) /
	      ((float) static_options->max_map_surface));
      target_shape.w /= f;
      target_shape.h /= f;
    }

  if (target_shape.w * target_shape.h < static_options->min_map_surface)
    {
      f =
	sqrt (((float) static_options->min_map_surface) /
	      (((float) target_shape.w) * ((float) target_shape.h)));
      target_shape.w *= f;
      target_shape.h *= f;
    }

  target_shape.w = lw6sys_min (target_shape.w, static_options->max_map_width);
  target_shape.h =
    lw6sys_min (target_shape.h, static_options->max_map_height);

  target_shape.w = lw6sys_max (target_shape.w, static_options->min_map_width);
  target_shape.h =
    lw6sys_max (target_shape.h, static_options->min_map_height);

  resampler->target_shape = target_shape;
  resampler->source_shape = (*source_shape);
  resampler->scale_x =
    ((float) (resampler->target_shape.w)) /
    ((float) (resampler->source_shape.w));
  resampler->scale_y =
    ((float) (resampler->target_shape.h)) /
    ((float) (resampler->source_shape.h));
}

static void
check_bounds (int *x, int *y, LW6SYS_WH * shape)
{
  (*x) = lw6sys_min ((*x), shape->w - 1);
  (*y) = lw6sys_min ((*y), shape->h - 1);
  (*x) = lw6sys_max ((*x), 0);
  (*y) = lw6sys_max ((*y), 0);
}

void
lw6map_resampler_source2target (LW6MAP_RESAMPLER * resampler, int *target_x,
				int *target_y, int source_x, int source_y)
{
  (*target_x) = (int) (source_x * resampler->scale_x);
  (*target_y) = (int) (source_y * resampler->scale_y);
  check_bounds (target_x, target_y, &(resampler->target_shape));
}

void
lw6map_resampler_target2source (LW6MAP_RESAMPLER * resampler, int *source_x,
				int *source_y, int target_x, int target_y)
{
  (*source_x) = (int) (target_x / resampler->scale_x);
  (*source_y) = (int) (target_y / resampler->scale_y);
  check_bounds (source_x, source_y, &(resampler->source_shape));
}
